@@ -3,8 +3,8 @@
  *
  * libyahoo2 is available at http://libyahoo2.sourceforge.net/
  *
- * $Revision: 1.72 $
- * $Date: 2004/01/16 05:38:46 $
+ * $Revision: 308 $
+ * $Date: 2008-04-08 07:09:21 -0700 (Tue, 08 Apr 2008) $
  * 
  * Copyright (C) 2002-2004, Philip S Tellis <philip.tellis AT gmx.net>
  *
@@ -27,33 +27,39 @@
 #if HAVE_CONFIG_H
 # include <config.h>
 #endif
+#ifndef _WIN32
 #include <netdb.h>
 #include <sys/time.h>
+#endif
 #include <sys/types.h>
+#ifndef _WIN32
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
 #include <termios.h>
+#endif
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <unistd.h>
 #include <time.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 /* Get these from http://libyahoo2.sourceforge.net/ */
 #include <yahoo2.h>
 #include <yahoo2_callbacks.h>
 #include "yahoo_util.h"
 
+#ifndef _WIN32
 int fileno(FILE * stream);
-
+#endif
 
 #define MAX_PREF_LEN 255
 
@@ -136,7 +142,7 @@ char * yahoo_status_code(enum yahoo_status s)
 
 #define YAHOO_DEBUGLOG ext_yahoo_log
 
-int ext_yahoo_log(char *fmt,...)
+int ext_yahoo_log(const char *fmt,...)
 {
 	va_list ap;
 
@@ -175,17 +181,20 @@ static void rearm(time_t *timer, int seconds)
 	*timer += seconds;
 }
 
+#ifndef _WIN32
 FILE *popen(const char *command, const char *type);
 int pclose(FILE *stream);
 int gethostname(char *name, size_t len);
+#endif
 
 static char * get_local_addresses()
 {
 	static char addresses[1024];
 	char buff[1024];
+	struct hostent * hn;
+#ifndef _WIN32
 	char gateway[16];
 	char  * c;
-	struct hostent * hn;
 	int i;
 	FILE * f;
 	f = popen("netstat -nr", "r");
@@ -237,6 +246,7 @@ static char * get_local_addresses()
 		
 		
 IP_TEST_2:
+#endif /* _WIN32 */
 
 	gethostname(buff,sizeof(buff));
 
@@ -251,11 +261,15 @@ IP_TEST_2:
 
 static double get_time()
 {
+#ifndef _WIN32
 	struct timeval ct;
 	gettimeofday(&ct, 0);
 
 	/* return time in milliseconds */
 	return (ct.tv_sec * 1E3 + ct.tv_usec / 1E3);
+#else
+	return timeGetTime();
+#endif
 }
 
 static int yahoo_ping_timeout_callback()
@@ -303,13 +317,14 @@ static int yahoo_webcam_timeout_callback(int id)
 YList * conferences = NULL;
 typedef struct {
 	int id;
+	char * me;
 	char * room_name;
 	char * host;
 	YList * members;
 	int joined;
 } conf_room;
 
-static char * get_buddy_name(char * yid)
+static const char * get_buddy_name(const char * yid)
 {
 	YList * b;
 	for (b = buddies; b; b = b->next) {
@@ -321,12 +336,12 @@ static char * get_buddy_name(char * yid)
 	return yid;
 }
 
-static conf_room * find_conf_room_by_name_and_id(int id, const char * name)
+static conf_room * find_conf_room_by_name_and_id(int id, const char * me, const char * name)
 {
 	YList * l;
 	for(l = conferences; l; l=l->next) {
 		conf_room * cr = l->data;
-		if(cr->id == id && !strcmp(name, cr->room_name)) {
+		if(cr->id == id && !strcmp(name, cr->room_name) && (me == NULL || cr->me == NULL || !strcmp(me, cr->me))) {
 			return cr;
 		}
 	}
@@ -334,28 +349,30 @@ static conf_room * find_conf_room_by_name_and_id(int id, const char * name)
 	return NULL;
 }
 
-void ext_yahoo_got_conf_invite(int id, char *who, char *room, char *msg, YList *members)
+void ext_yahoo_got_conf_invite(int id, const char *me, const char *who, const char *room, const char *msg, YList *members)
 {
 	conf_room * cr = y_new0(conf_room, 1);
 	cr->room_name = strdup(room);
+	cr->me = strdup(me);
 	cr->host = strdup(who);
 	cr->members = members;
 	cr->id = id;
 
 	conferences = y_list_append(conferences, cr);
 
-	print_message(("%s has invited you to a conference: %s\n"
+	print_message(("%s has invited you [%s] to a conference: %s\n"
 				"with the message: %s",
-				who, room, msg));
+				who, me, room, msg));
 
 	for(; members; members=members->next)
 		print_message(("\t%s", (char *)members->data));
 	
 }
-void ext_yahoo_conf_userdecline(int id, char *who, char *room, char *msg)
+void ext_yahoo_conf_userdecline(int id, const char *me, const char *who, const char *room, const char *msg)
 {
 	YList * l;
-	conf_room * cr = find_conf_room_by_name_and_id(id, room);
+	/* TODO: probably have to use the me arg to find the room */
+	conf_room * cr = find_conf_room_by_name_and_id(id, me, room);
 	
 	if(cr)
 	for(l = cr->members; l; l=l->next) {
@@ -368,12 +385,12 @@ void ext_yahoo_conf_userdecline(int id, char *who, char *room, char *msg)
 		}
 	}
 
-	print_message(("%s declined the invitation to %s\n"
-				"with the message: %s", who, room, msg));
+	print_message(("%s declined the invitation from %s to %s\n"
+				"with the message: %s", who, me, room, msg));
 }
-void ext_yahoo_conf_userjoin(int id, char *who, char *room)
+void ext_yahoo_conf_userjoin(int id, const char *me, const char *who, const char *room)
 {
-	conf_room * cr = find_conf_room_by_name_and_id(id, room);
+	conf_room * cr = find_conf_room_by_name_and_id(id, me, room);
 	if(cr) {
 	YList * l = NULL;
 	for(l = cr->members; l; l=l->next) {
@@ -385,13 +402,13 @@ void ext_yahoo_conf_userjoin(int id, char *who, char *room)
 		cr->members = y_list_append(cr->members, strdup(who));
 	}
 
-	print_message(("%s joined the conference %s", who, room));
+	print_message(("%s joined the conference %s [%s]", who, room, me));
 
 }
-void ext_yahoo_conf_userleave(int id, char *who, char *room)
+void ext_yahoo_conf_userleave(int id, const char *me, const char *who, const char *room)
 {
 	YList * l;
-	conf_room * cr = find_conf_room_by_name_and_id(id, room);
+	conf_room * cr = find_conf_room_by_name_and_id(id, me, room);
 	
 	if(cr)
 	for(l = cr->members; l; l=l->next) {
@@ -404,18 +421,18 @@ void ext_yahoo_conf_userleave(int id, char *who, char *room)
 		}
 	}
 
-	print_message(("%s left the conference %s", who, room));
+	print_message(("%s left the conference %s [%s]", who, room, me));
 }
-void ext_yahoo_conf_message(int id, char *who, char *room, char *msg, int utf8)
+void ext_yahoo_conf_message(int id, const char *me, const char *who, const char *room, const char *msg, int utf8)
 {
-	char * umsg = msg;
+	char * umsg = (char *)msg;
 
 	if(utf8)
 		umsg = y_utf8_to_str(msg);
 
 	who = get_buddy_name(who);
 
-	print_message(("%s (in %s): %s", who, room, umsg));
+	print_message(("%s (in %s [%s]): %s", who, room, me, umsg));
 
 	if(utf8)
 		FREE(umsg);
@@ -439,14 +456,14 @@ static void print_chat_member(struct yahoo_chat_member *ycm)
 	printf("  Location: %s", ycm->location);
 }
 
-void ext_yahoo_chat_cat_xml(int id, char *xml) 
+void ext_yahoo_chat_cat_xml(int id, const char *xml) 
 {
 	print_message(("%s", xml));
 }
 
-void ext_yahoo_chat_join(int id, char *room, char * topic, YList *members, int fd)
+void ext_yahoo_chat_join(int id, const char *me, const char *room, const char * topic, YList *members, int fd)
 {
-	print_message(("You have joined the chatroom %s with topic %s", room, topic));
+	print_message(("You [%s] have joined the chatroom %s with topic %s", me, room, topic));
 
 	while(members) {
 		YList *n = members->next;
@@ -462,22 +479,22 @@ void ext_yahoo_chat_join(int id, char *room, char * topic, YList *members, int f
 		members=n;
 	}
 }
-void ext_yahoo_chat_userjoin(int id, char *room, struct yahoo_chat_member *who)
+void ext_yahoo_chat_userjoin(int id, const char *me, const char *room, struct yahoo_chat_member *who)
 {
 	print_chat_member(who);
-	print_message((" joined the chatroom %s", room));
+	print_message((" joined the chatroom %s [%s]", room, me));
 	FREE(who->id);
 	FREE(who->alias);
 	FREE(who->location);
 	FREE(who);
 }
-void ext_yahoo_chat_userleave(int id, char *room, char *who)
+void ext_yahoo_chat_userleave(int id, const char *me, const char *room, const char *who)
 {
-	print_message(("%s left the chatroom %s", who, room));
+	print_message(("%s left the chatroom %s [%s]", who, room, me));
 }
-void ext_yahoo_chat_message(int id, char *who, char *room, char *msg, int msgtype, int utf8)
+void ext_yahoo_chat_message(int id, const char *me, const char *who, const char *room, const char *msg, int msgtype, int utf8)
 {
-	char * umsg = msg;
+	char * umsg = (char *)msg;
 	char * charpos;
 
 	if(utf8)
@@ -492,19 +509,21 @@ void ext_yahoo_chat_message(int id, char *who, char *room, char *msg, int msgtyp
 	}
 
 	if (msgtype == 2) {
-		print_message(("(in %s) %s %s", room, who, umsg));
+		print_message(("(in %s [%s]) %s %s", room, me, who, umsg));
 	} else {
-		print_message(("(in %s) %s: %s", room, who, umsg));
+		print_message(("(in %s [%s]) %s: %s", room, me, who, umsg));
 	}
 
 	if(utf8)
 		FREE(umsg);
 }
 
-void ext_yahoo_status_changed(int id, char *who, int stat, char *msg, int away)
+void ext_yahoo_status_changed(int id, const char *who, int stat, const char *msg, int away, int idle, int mobile)
 {
 	yahoo_account * ya=NULL;
 	YList * b;
+	char buf[1024];
+
 	for(b = buddies; b; b = b->next) {
 		if(!strcmp(((yahoo_account *)b->data)->yahoo_id, who)) {
 			ya = b->data;
@@ -512,14 +531,34 @@ void ext_yahoo_status_changed(int id, char *who, int stat, char *msg, int away)
 		}
 	}
 	
-	if(msg)
-		print_message(("%s (%s) is now %s", ya?ya->name:who, who, msg))
-	else if(stat == YAHOO_STATUS_IDLE)
-		print_message(("%s (%s) idle for %d:%02d:%02d", ya?ya->name:who, who, 
-					away/3600, (away/60)%60, away%60))
-	else
-		print_message(("%s (%s) is now %s", ya?ya->name:who, who, 
-					yahoo_status_code(stat)))
+	if (msg == NULL) {
+		sprintf(buf, "%s", yahoo_status_code(stat));
+	}
+	else if (stat == YAHOO_STATUS_CUSTOM) {
+		sprintf(buf, "%s", msg);
+	} else {
+		sprintf(buf, "%s: %s", yahoo_status_code(stat), msg);
+	}
+
+	if (away > 0) {
+		char away_buf[32];
+		sprintf(away_buf, " away[%d]", away);
+		strcat(buf, away_buf);
+	}
+
+	if (mobile > 0) {
+		char mobile_buf[32];
+		sprintf(mobile_buf, " mobile[%d]", mobile);
+		strcat(buf, mobile_buf);
+	}
+
+	if (idle > 0) {
+		char time_buf[32];
+		sprintf(time_buf, " idle for %d:%02d:%02d", idle/3600, (idle/60)%60, idle%60);
+		strcat(buf, time_buf);
+	}
+
+	print_message(("%s (%s) is now %s", ya?ya->name:who, who, buf))
 
 	if(ya) {
 		ya->status = stat;
@@ -557,12 +596,12 @@ void ext_yahoo_got_ignore(int id, YList * igns)
 {
 }
 
-void ext_yahoo_got_im(int id, char *who, char *msg, long tm, int stat, int utf8)
+void ext_yahoo_got_im(int id, const char *me, const char *who, const char *msg, long tm, int stat, int utf8)
 {
-	char *umsg = msg;
+	char *umsg = (char *)msg;
 
 	if(stat == 2) {
-		LOG(("Error sending message to %s", who));
+		LOG(("Error sending message from %s to %s", me, who));
 		return;
 	}
 
@@ -580,26 +619,26 @@ void ext_yahoo_got_im(int id, char *who, char *msg, long tm, int stat, int utf8)
 		strncpy(timestr, ctime((time_t *)&tm), sizeof(timestr));
 		timestr[strlen(timestr) - 1] = '\0';
 
-		print_message(("[Offline message at %s from %s]: %s", 
-				timestr, who, umsg))
+		print_message(("[Offline message at %s to %s from %s]: %s", 
+				timestr, me, who, umsg))
 	} else {
 		if(!strcmp(umsg, "<ding>")) 
 			printf("\a");
-		print_message(("%s: %s", who, umsg))
+		print_message(("[%s]%s: %s", me, who, umsg))
 	}
 
 	if(utf8)
 		FREE(umsg);
 }
 
-void ext_yahoo_rejected(int id, char *who, char *msg)
+void ext_yahoo_rejected(int id, const char *who, const char *msg)
 {
 	print_message(("%s has rejected you%s%s", who, 
 				(msg?" with the message:\n":"."), 
 				(msg?msg:"")));
 }
 
-void ext_yahoo_contact_added(int id, char *myid, char *who, char *msg)
+void ext_yahoo_contact_added(int id, const char *myid, const char *who, const char *msg)
 {
 	char buff[1024];
 
@@ -620,17 +659,17 @@ void ext_yahoo_contact_added(int id, char *myid, char *who, char *msg)
 */
 }
 
-void ext_yahoo_typing_notify(int id, char *who, int stat)
+void ext_yahoo_typing_notify(int id, const char* me, const char *who, int stat)
 {
 	if(stat && do_typing_notify)
-		print_message(("%s is typing...", who));
+		print_message(("[%s]%s is typing...", me, who));
 }
 
-void ext_yahoo_game_notify(int id, char *who, int stat)
+void ext_yahoo_game_notify(int id, const char *me, const char *who, int stat)
 {
 }
 
-void ext_yahoo_mail_notify(int id, char *from, char *subj, int cnt)
+void ext_yahoo_mail_notify(int id, const char *from, const char *subj, int cnt)
 {
 	char buff[1024] = {0};
 	
@@ -694,7 +733,7 @@ void ext_yahoo_got_webcam_image(int id, const char *who,
 	}
 }
 
-void ext_yahoo_webcam_viewer(int id, char *who, int connect)
+void ext_yahoo_webcam_viewer(int id, const char *who, int connect)
 {
 	switch (connect)
 	{
@@ -711,7 +750,7 @@ void ext_yahoo_webcam_viewer(int id, char *who, int connect)
 	}
 }
 
-void ext_yahoo_webcam_closed(int id, char *who, int reason)
+void ext_yahoo_webcam_closed(int id, const char *who, int reason)
 {
 	switch(reason)
 	{
@@ -744,21 +783,21 @@ void ext_yahoo_webcam_data_request(int id, int send)
 	send_webcam_images = send;
 }
 
-void ext_yahoo_webcam_invite(int id, char *from)
+void ext_yahoo_webcam_invite(int id, const char *me, const char *from)
 {
-	print_message(("Got a webcam invitation from %s", from));
+	print_message(("Got a webcam invitation to %s from %s", me, from));
 }
 
-void ext_yahoo_webcam_invite_reply(int id, char *from, int accept)
+void ext_yahoo_webcam_invite_reply(int id, const char *me, const char *from, int accept)
 {
 	if(accept) {
-		print_message(("%s accepted webcam invitation...", from));
+		print_message(("[%s]%s accepted webcam invitation...", me, from));
 	} else {
-		print_message(("%s declined webcam invitation...", from));
+		print_message(("[%s]%s declined webcam invitation...", me, from));
 	}
 }
 
-void ext_yahoo_system_message(int id, char *msg)
+void ext_yahoo_system_message(int id, const char *msg)
 {
 	if(ignore_system)
 		return;
@@ -779,6 +818,7 @@ void yahoo_logout()
 		conf_room * cr = conferences->data;
 		if(cr->joined)
 			yahoo_conference_logoff(ylad->id, NULL, cr->members, cr->room_name);
+		FREE(cr->me);
 		FREE(cr->room_name);
 		FREE(cr->host);
 		while(cr->members) {
@@ -827,7 +867,7 @@ void ext_yahoo_got_cookies(int id)
 	/*yahoo_get_yab(id);*/
 }
 
-void ext_yahoo_login_response(int id, int succ, char *url)
+void ext_yahoo_login_response(int id, int succ, const char *url)
 {
 	char buff[1024];
 
@@ -863,12 +903,17 @@ void ext_yahoo_login_response(int id, int succ, char *url)
 	poll_loop=0;
 }
 
-void ext_yahoo_error(int id, char *err, int fatal)
+void ext_yahoo_error(int id, const char *err, int fatal, int num)
 {
 	fprintf(stdout, "Yahoo Error: ");
 	fprintf(stdout, "%s", err);
-	/*
 	switch(num) {
+		case E_UNKNOWN:
+			fprintf(stdout, "unknown error %s", err);
+			break;
+		case E_CUSTOM:
+			fprintf(stdout, "custom error %s", err);
+			break;
 		case E_CONFNOTAVAIL:
 			fprintf(stdout, "%s is not available for the conference", err);
 			break;
@@ -881,8 +926,13 @@ void ext_yahoo_error(int id, char *err, int fatal)
 		case E_IGNORECONF:
 			fprintf(stdout, "%s is in buddy list - cannot ignore ", err);
 			break;
+		case E_SYSTEM:
+			fprintf(stdout, "system error %s", err);
+			break;
+		case E_CONNECTION:
+			fprintf(stdout, "server connection error %s", err);
+			break;
 	}
-	*/
 	fprintf(stdout, "\n");
 	if(fatal)
 		yahoo_logout();
@@ -908,7 +958,7 @@ void yahoo_set_current_state(int yahoo_state)
 		yahoo_set_away(ylad->id, yahoo_state, NULL, 1);
 }
 
-int ext_yahoo_connect(char *host, int port)
+int ext_yahoo_connect(const char *host, int port)
 {
 	struct sockaddr_in serv_addr;
 	static struct hostent *server;
@@ -1055,7 +1105,7 @@ void yahoo_callback(struct _conn *c, yahoo_input_condition cond)
 	}
 }
 
-int ext_yahoo_connect_async(int id, char *host, int port, 
+int ext_yahoo_connect_async(int id, const char *host, int port, 
 		yahoo_connect_callback callback, void *data)
 {
 	struct sockaddr_in serv_addr;
@@ -1133,11 +1183,11 @@ static void process_commands(char *line)
 		msg = copy;
 		if(to && msg) {
 			if(!strcmp(msg, "\a"))
-				yahoo_send_im(ylad->id, NULL, to, "<ding>", 0);
+				yahoo_send_im(ylad->id, NULL, to, "<ding>", 0,0);
 			else {
 				msg = y_str_to_utf8(msg);
-				yahoo_send_im(ylad->id, NULL, to, msg, 1);
-				FREE(msg);
+				yahoo_send_im(ylad->id, NULL, to, msg, 1,0);
+				FREE(msg)
 			}
 		}
 	} else if(!strncasecmp(cmd, "CMS", strlen("CMS"))) {
@@ -1150,7 +1200,7 @@ static void process_commands(char *line)
 			copy = tmp+1;
 		}
 		msg = copy;
-		cr = find_conf_room_by_name_and_id(ylad->id, to);
+		cr = find_conf_room_by_name_and_id(ylad->id, NULL, to);
 		if(!cr) {
 			print_message(("no such room: %s", copy));
 			goto end_parse;
@@ -1160,7 +1210,7 @@ static void process_commands(char *line)
 	} else if(!strncasecmp(cmd, "CLS", strlen("CLS"))) {
 		YList * l;
 		if(copy) {
-			conf_room * cr = find_conf_room_by_name_and_id(ylad->id, copy);
+			conf_room * cr = find_conf_room_by_name_and_id(ylad->id, NULL, copy);
 			if(!cr) {
 				print_message(("no such room: %s", copy));
 				goto end_parse;
@@ -1210,7 +1260,7 @@ static void process_commands(char *line)
 			l = y_list_append(l, copy);
 		}
 
-		cr = find_conf_room_by_name_and_id(ylad->id, room);
+		cr = find_conf_room_by_name_and_id(ylad->id, NULL, room);
 		if(!cr) {
 			print_message(("no such room: %s", room));
 			y_list_free(l);
@@ -1225,7 +1275,7 @@ static void process_commands(char *line)
 		y_list_free(l);
 
 	} else if(!strncasecmp(cmd, "CLN", strlen("CLN"))) {
-		conf_room * cr = find_conf_room_by_name_and_id(ylad->id, copy);
+		conf_room * cr = find_conf_room_by_name_and_id(ylad->id, NULL, copy);
 		YList * l;
 		if(!cr) {
 			print_message(("no such room: %s", copy));
@@ -1243,7 +1293,7 @@ static void process_commands(char *line)
 		yahoo_conference_logon(ylad->id, NULL, cr->members, copy);
 
 	} else if(!strncasecmp(cmd, "CLF", strlen("CLF"))) {
-		conf_room * cr = find_conf_room_by_name_and_id(ylad->id, copy);
+		conf_room * cr = find_conf_room_by_name_and_id(ylad->id, NULL, copy);
 		
 		if(!cr) {
 			print_message(("no such room: %s", copy));
@@ -1275,7 +1325,7 @@ static void process_commands(char *line)
 			msg = "Thanks, but no thanks!";
 		}
 		
-		cr = find_conf_room_by_name_and_id(ylad->id, room);
+		cr = find_conf_room_by_name_and_id(ylad->id, NULL, room);
 		if(!cr) {
 			print_message(("no such room: %s", room));
 			goto end_parse;
@@ -1417,6 +1467,7 @@ end_parse:
 	FREE(start);
 }
 
+#ifndef _WIN32
 static void local_input_callback(int source)
 {
 	char line[1024] = {0};
@@ -1445,6 +1496,34 @@ static void local_input_callback(int source)
 	if(line[0])
 		process_commands(line);
 }
+#else
+#include <conio.h>
+static void local_input_callback(char c)
+{
+	static char line[1024] = {0};
+	static int line_length = 0;
+
+	if (c == '\b' || (int)c == 127) {
+		if (line_length > 0) {
+			_cputs("\b \b");
+			line_length--;
+		}
+		return;
+	}
+
+	if (c == '\n' || c == '\r' || c == 3) {
+		_cputs("\n");
+		line[line_length] = 0;
+		process_commands(line);
+		line_length = 0;
+		line[0] = 0;
+		return;
+	}
+
+	_putch(c);
+	line[line_length++] = c;
+}
+#endif
 
 int main(int argc, char * argv[])
 {
@@ -1455,10 +1534,17 @@ int main(int argc, char * argv[])
 	fd_set inp, outp;
 	struct timeval tv;
 
-
+#ifndef _WIN32
 	int fd_stdin = fileno(stdin);
+#endif
 	YList *l=connections;
 
+#ifdef _WIN32
+	WSADATA wsa;	
+	if (WSAStartup(MAKEWORD(2,2), &wsa))
+		return -1;
+#endif
+
 	ylad = y_new0(yahoo_local_account, 1);
 
 	local_host = strdup(get_local_addresses());
@@ -1466,6 +1552,9 @@ int main(int argc, char * argv[])
 	printf("Yahoo Id: ");
 	scanf("%s", ylad->yahoo_id);
 	printf("Password: ");
+#ifdef _WIN32
+	scanf("%s", ylad->password);
+#else
 	{
 		tcflag_t oflags;
 		struct termios term;
@@ -1482,6 +1571,7 @@ int main(int argc, char * argv[])
 		tcsetattr(fd_stdin, TCSANOW, &term);
 	}
 	printf("\n");
+#endif
 
 	printf("Initial Status: ");
 	scanf("%d", &status);
@@ -1498,9 +1588,14 @@ int main(int argc, char * argv[])
 	while(poll_loop) {
 		FD_ZERO(&inp);
 		FD_ZERO(&outp);
+#ifndef _WIN32
 		FD_SET(fd_stdin, &inp);
 		tv.tv_sec=1;
 		tv.tv_usec=0;
+#else
+		tv.tv_sec=0;
+		tv.tv_usec=1E4;
+#endif
 		lfd=0;
 
 		for(l=connections; l; ) {
@@ -1526,7 +1621,11 @@ int main(int argc, char * argv[])
 		select(lfd + 1, &inp, &outp, NULL, &tv);
 		time(&curTime);
 
+#ifndef _WIN32
 		if(FD_ISSET(fd_stdin, &inp))	local_input_callback(0);
+#else
+		if (_kbhit()) local_input_callback(_getch());
+#endif
 
 		for(l = connections; l; l = y_list_next(l)) {
 			struct _conn *c = l->data;
@@ -1556,23 +1655,31 @@ int main(int argc, char * argv[])
 
 	FREE(ylad);
 
+#ifdef _WIN32
+	WSACleanup();
+#endif
 	return 0;
 }
 
-void ext_yahoo_got_file(int id, char *who, char *url, long expires, char *msg, char *fname, unsigned long fesize)
+void ext_yahoo_got_file(int id, const char *me, const char *who, const char *url, long expires, const char *msg, const char *fname, unsigned long fesize)
 {
 }
 
 void ext_yahoo_got_identities(int id, YList * ids)
 {
 }
-void ext_yahoo_chat_yahoologout(int id)
+void ext_yahoo_chat_yahoologout(int id, const char *me)
 { 
- 	LOG(("got chat logout"));
+ 	LOG(("got chat logout for %s", me));
 }
-void ext_yahoo_chat_yahooerror(int id)
+void ext_yahoo_chat_yahooerror(int id, const char *me)
 { 
- 	LOG(("got chat logout"));
+ 	LOG(("got chat error for %s", me));
+}
+
+void ext_yahoo_got_ping(int id, const char *errormsg)
+{ 
+ 	LOG(("got ping errormsg %s", errormsg));
 }
 
 void ext_yahoo_got_search_result(int id, int found, int start, int total, YList *contacts)
@@ -1580,6 +1687,26 @@ void ext_yahoo_got_search_result(int id, int found, int start, int total, YList
 	LOG(("got search result"));
 }
 
+void ext_yahoo_got_buddyicon_checksum(int id, const char *a, const char *b, int checksum)
+{
+	LOG(("got buddy icon checksum"));
+}
+
+void ext_yahoo_got_buddyicon(int id, const char *a, const char *b, const char *c, int checksum)
+{
+	LOG(("got buddy icon"));
+}
+
+void ext_yahoo_buddyicon_uploaded(int id, const char *url)
+{
+	LOG(("buddy icon uploaded"));
+}
+
+void ext_yahoo_got_buddyicon_request(int id, const char *me, const char *who)
+{
+	LOG(("got buddy icon request from %s",who));
+}
+
 static void register_callbacks()
 {
 #ifdef USE_STRUCT_CALLBACKS
@@ -1624,6 +1751,11 @@ static void register_callbacks()
 	yc.ext_yahoo_remove_handler = ext_yahoo_remove_handler;
 	yc.ext_yahoo_connect = ext_yahoo_connect;
 	yc.ext_yahoo_connect_async = ext_yahoo_connect_async;
+	yc.ext_yahoo_got_buddyicon = ext_yahoo_got_buddyicon;
+	yc.ext_yahoo_got_buddyicon_checksum = ext_yahoo_got_buddyicon_checksum;
+	yc.ext_yahoo_buddyicon_uploaded = ext_yahoo_buddyicon_uploaded;
+	yc.ext_yahoo_got_buddyicon_request = ext_yahoo_got_buddyicon_request;
+	yc.ext_yahoo_got_ping = ext_yahoo_got_ping;
 
 	yahoo_register_callbacks(&yc);
 	
